unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, SyncObjs, wsNetSendLib, ExtCtrls, ComCtrls;

const
  CM_NEWMSG = WM_USER + 1;

type
  TMsgBufItem = record
    from: string;
    addr :string;
    msgtext: string;
  end;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    Button2: TButton;
    Edit1: TEdit;
    Button4: TButton;
    Memo2: TMemo;
    Edit2: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label5: TLabel;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    Button3: TButton;
    Timer1: TTimer;
    StatusBar1: TStatusBar;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button4Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    msgBufLock: TCriticalSection;
    msgBuf: TList;
    procedure OnNewMsg(var Message:TMessage);message CM_NEWMSG;
  public
    { Public declarations }
    procedure NewMsg( aFrom, aTo, aMsg: string );
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure NewMsgCallback( const aFrom, aTo, aMsg: LPCTSTR ); stdcall;
begin
  Form1.NewMsg(aFrom, aTo, aMsg);
end;

function GetStateName( aState: integer ): string;
begin
  case aState of
    WSNS_STATE_NONETWORK: result := 'Network not found';
    WSNS_STATE_INACTIVE: result := 'Offline';
    WSNS_STATE_ACTIVATION: result := 'Registering...';
    WSNS_STATE_ACTIVE: result := 'Online';
  else
    result := '?';
  end;
end;

function GetComputerName(): string;
var
	compName: array [0..MAX_COMPUTERNAME_LENGTH + 1] of char;
  compNameSize:DWORD;
begin
  compNameSize := MAX_COMPUTERNAME_LENGTH + 1;
	Windows.GetComputerName(compName, &compNameSize);
	CharUpper(compName);

	result := compName;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  msgBufLock := TCriticalSection.Create();
  msgBuf := TList.Create();

  NetsendBroadcastInitialize();
  NetsendUnicastInitialize();

  Edit1.Text := GetComputerName();
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  NetsendBroadcastDeactivate();
  NetsendUnicastDeactivate();
  
  NetsendBroadcastFinalize();
  NetsendUnicastFinalize();
end;

procedure TForm1.NewMsg( aFrom, aTo, aMsg: string );
var
  msg: ^TMsgBufItem;
begin
  New(msg);
  msg.from := aFrom;
  msg.addr := aTo;
  msg.msgtext := aMsg;

  msgBufLock.Acquire();
  msgBuf.Add( msg );
  msgBufLock.Release();

  PostMessage(WindowHandle, CM_NEWMSG, 0, 0);
end;

procedure TForm1.OnNewMsg(var Message:TMessage);
var
  msg: ^TMsgBufItem;
  i: integer;
begin
  msgBufLock.Acquire();

  for i:=0 to msgBuf.Count-1 do
  begin
    msg := msgBuf[i];

    memo1.Lines.Add('From: ' + msg.from);
    memo1.Lines.Add('To: '  + msg.addr);
    memo1.Lines.Add(msg.msgtext);
    memo1.Lines.Add('');

    Dispose(msgBuf[i]);
  end;
  msgBuf.Clear();
  
  msgBufLock.Release();
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  NetsendBroadcastSetOnNewMsg( NewMsgCallback );
  NetsendBroadcastActivate();

  NetsendUnicastAddName( PChar(Edit1.Text) );
  NetsendUnicastSetOnNewMsg( NewMsgCallback );
  NetsendUnicastActivate();

  Edit1.Enabled := false;
  Button1.Enabled := false;
  Button2.Enabled := true;
  Button4.Enabled := true;
  Memo1.Enabled := true;
  Edit2.Enabled := true;
  RadioButton1.Enabled := true;
  RadioButton2.Enabled := true;
  Memo2.Enabled := true;
end;


procedure TForm1.Button2Click(Sender: TObject);
begin
  NetsendBroadcastDeactivate();
  NetsendUnicastDeactivate();

  Edit1.Enabled := true;
  Button1.Enabled := true;
  Button2.Enabled := false;
  Button4.Enabled := false;
  Memo1.Enabled := false;
  Edit2.Enabled := false;
  RadioButton1.Enabled := false;
  RadioButton2.Enabled := false;
  Memo2.Enabled := false;
end;

procedure TForm1.Button4Click(Sender: TObject);
var
  r: integer;
begin

  if ( RadioButton1.Checked ) then
    r := NetsendUnicastSendMsgEx( PChar(Edit2.Text), PChar(Memo2.Text), PChar(Edit1.Text) )
  else
    r := NetsendBroadcastSendMsgEx( PChar(Edit2.Text), PChar(Memo2.Text), PChar(Edit1.Text) );

  if ( r <> WSNS_RESULT_SUCCESS ) then
    MessageDlg('Unknown error', mtError, [mbOK], 0)
  else
    Memo2.Clear();
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  Close();
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  stateb, stateu: DWORD;
  stnameu, stnameb: string;
begin
  if ( NetsendUnicastGetState( PChar(Edit1.Text), @stateu ) = WSNS_RESULT_SUCCESS ) then
  begin
    stnameu := GetStateName(stateu);
    StatusBar1.Panels[0].Text := 'Personal name (' + Edit1.Text + ') - ' + stnameu;
  end;

  if ( NetsendBroadcastGetState( @stateb ) = WSNS_RESULT_SUCCESS ) then
  begin
    stnameb := GetStateName(stateb);
    StatusBar1.Panels[1].Text := 'Group name (*) - ' +stnameb;
  end;
end;

end.
